/*
 * XLife Copyright 1989 Jon Bennett jb7m+@andrew.cmu.edu, jcrb@cs.cmu.edu
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holders make no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#ifdef __GNUCC__
; /* bogus semi-colon to appease the GNU god */
#endif /* __GNUCC__ */
#include <stdio.h>
#include <X11/Xos.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#ifndef SMALLFONT
#define NORMALFONT "9x15"
#define BOLDFONT "9x15bold"
#define FONTHEIGHT 15
#define FONTWIDTH 9
#else
#define NORMALFONT "8x13"
#define BOLDFONT "8x13bold"
#define FONTHEIGHT 13
#define FONTWIDTH 8
#endif

#define BORDERWIDTH 0

#define MAINWIN 0

#define LIFEWIN 1	/* the viewport on the Life universe */

#define INPUTWIN 2	/* the command-input window */
#define INPUTXOFF 2
#define INPUTH 20
#define INPUTYOFF FONTHEIGHT
#define INPUTLEN 125
#define INPUTTEXTLEN (INPUTLEN - INPUTFROMLEN)
#define ICOORDS(x,y) (INPUTXOFF + (x) * FONTWIDTH), (INPUTYOFF + (y) * FONTHEIGHT)

#define RULEWIN	3	/* the rule window */
#define RULEXOFF 2
#define RULEYOFF FONTHEIGHT
#define CWRULES(x,y) (RULEXOFF + (x) * FONTWIDTH), (RULEYOFF + (y) * FONTHEIGHT)
#define RULEW (FONTWIDTH * 16)

#define COORDWIN 4	/* the coordinate-display window */
#define COORDXOFF 2
#define COORDYOFF FONTHEIGHT
#define CWCOORDS(x,y) (COORDXOFF + (x) * FONTWIDTH), (COORDYOFF + (y) * FONTHEIGHT)
#define COORDW 200

#define HELPWIN 5	/* the help window */

/* mouse coordinates to life-universe coordinates */
#define XPOS(x,xpos) (shr((unsigned)(x), scale) + xpos)
#define YPOS(y,ypos) (shr((unsigned)(y), scale) + ypos)

/* life-universe coordinates to mouse coordinates */
#define RXPOS(x,xpos) shl(((x) - (xpos)), scale)
#define RYPOS(y,ypos) shl(((y) - (ypos)), scale)

/* mouse coordinates to cell coords (relative to top left of screen) */
#define MOUSE2CELL(n)	((n) & shl(0xffffffff, scale))

/* this magic number is roughly halfway through the range of unsigned long */
#define STARTX 2100000000
#define STARTY 2100000000

/* the program's run states */
#define STOP 	0x0	/* single-stepping */
#define HIDE	0x1	/* evolving, no per-generation display */
#define RUN	0x2	/* evolving with per-generation display */

/* time-delays for generation evolution */
#define DELAY_FAST 0
#define DELAY_MED 250
#define DELAY_SLOW 500
#define DELAY_INCREMENT 50

#define DEADTIME	10	/* nuke a tile when it's been dead this long */

#define PATCH_LOG	"new.transitions"

#define LOADEXT		".l"
#define LOADEXTLEN	2

typedef int	bool;
#define TRUE	1
#define FALSE	0

/*
 * If you don't want n-state capability, define this to 1 in the Makefile
 * to save some storage and reduce the program size.
 */
#ifndef STATEBITS
#define STATEBITS	3
#endif /* STATEBITS */

#define MAXSTATES	(1<<STATEBITS)
#define BADSTATE	0xff	/* out-of-band cell-state value */

#if STATEBITS <= 3
#define is_state(c)	isdigit(c)	/* is char a valid state indicator? */
#define stoi(c)		((c) - '0')	/* state character to int */
#define itos(n)		('0' + (n))	/* int to state character */
#else /* max is 4 */
bool is_state(char);
int stoi(char);
char itos(int);
#endif

#if STATEBITS <= 8
typedef unsigned char	cell_t;		/* allows 256 states per cell */
#elif STATEBITS <= 16
typedef short	cell_t;			/* allows 65536 states per cell */
#else
typedef int	cell_t;			/* are you serious? */
#endif

/*
 * These typedefs might change on 64-bit machines like the DEC Alpha to
 * support *really large* universes.  Ideally, cellcount_t should be
 * twice the length of coord_t, since it might hold coord_t's max squared.
 * It's hardly likely it will ever go that high, though...
 */
typedef unsigned long	coord_t;	/* hold world coordinates */
typedef unsigned long	cellcount_t;	/* hold a cell count */

#define PATNAMESIZ 128 /* It's an arbitrary limit, but probably enough */

typedef struct lq {
    unsigned long loadtime;
    char patname[PATNAMESIZ];
    int relpath;
    coord_t hotx, hoty;
    int rxx, rxy, ryx, ryy;
    struct lq  *next;
} LoadReq;

typedef struct {int xi; int yi;} offsets;

/* definitions for geometric transformations */ 

/* Get around "%" bogosity, and compute mod *correctly*. */
#define mod(m,n) (((m)>=0)?((m)%(n)):((n)-1-(-(m)+(n)-1)%(n)))

#define max(x, y)	(((x) > (y)) ? (x) : (y))
#define min(x, y)	(((x) > (y)) ? (y) : (x))
#define abs(x)	  	(((x) > (y)) ? ((x) - (y)) : ((y) - (x)))
#define shr(x,n)	(((n) < 0) ? ((x) << -(n)) : ((x) >> (n)))
#define shl(x,n)	(((n) < 0) ? ((x) >> -(n)) : ((x) << (n)))

/* compute coordinate transform for rotations */
/* yes, macros are equivalent, but this makes the meaning clearer (maybe) */
#define tx(x,y,rxx,rxy) ((rxx)*(x)+(rxy)*(y))
#define ty(x,y,ryx,ryy) ((ryx)*(x)+(ryy)*(y))

#ifdef SVR4
#define srandom(n)	srand48(n);
#define random()	lrand48()
#endif

/* tile.c */
#define BOXSIZE         8

/* someday, on a 64-bit machine, this might be unsigned int rather than long */
/* someday was years ago, catch up */
typedef unsigned int    u32bits;

typedef union {
    struct twostate_t {
        u32bits live1,live2,olive1,olive2;
        u32bits on[BOXSIZE];
    } twostate; /* 48 bytes (on a 32-bit machine) */
#if STATEBITS > 1
    struct nstate_t {
        cell_t  cell[BOXSIZE][BOXSIZE];
        cell_t  ocell[BOXSIZE][BOXSIZE];
    } nstate;   /* 128 bytes (assuming char-sized cells) */
#endif /* STATEBITS */
} cellbox;

typedef struct tile_t {
    coord_t x,y;
    int dead;
    struct tile_t *up, *dn, *lf, *rt, *fore, *next,*hfore,*hnext;

    /* MUST be last in the structure or our dynamic allocation will fail */
    cellbox cells;
} tile;

/*
 * A `pattern' is a linked list of tiles with its own context globals.
 * The universe may contain multiple patterns; they are overlayed on
 * the screen, but can be manipulated independently and evolve independently.
 */
typedef struct {
    tile                *tiles;                 /* head of the cell list */
    coord_t             tilecount;              /* count of boxes */
    cellcount_t         cellcount[MAXSTATES];   /* count of cells */
    unsigned long       generations;            /* context generation number */

#define HASHSIZE        32768
    tile                *hashlist[HASHSIZE];    /* hash list */
    coord_t             xmax, xmin, ymax, ymin;
    char                *pattern_name;
} pattern;


/* file.c */
void fileinit(void);
void savefile(void);
void do_loadreq(LoadReq*, pattern*);
void add_loadreq();
void free_loadscript(void);
void saveloadscript(void);
void confirmload(void);
void undoload(void);
void genload(void);
void loadfile(void);
char* use_rules(char *cp);
void loadrules(void);

/* file_misc.c */
void name_file(void);
void comment(void);
void view_comments(void);

/* gentab.c */
void gentab(void);

/* help.c */
void help(void);
void redraw_help(void);

/* key.c */
void getxstring(void);
void set_transition(void);
void test_transition(void);
int patch_transition(cell_t, cell_t, cell_t, cell_t, cell_t);

/* main.c */
void Motion(void), DoResize(void), DoExpose(const int);
void alloc_states(unsigned int);

/* tentative.c */
void moveload(void);
void flipload(void);
void turnload(void);
void boxpattern(int color);
void make_tentative(coord_t, coord_t, coord_t, coord_t);

/* tile.c declarations are in tile.h */

/* utils.c */
void announce(const char*);
void fatal(const char*);
int ClassifyWin(Window);
char *itoa(int);
void drawcell(coord_t, coord_t, cell_t);
void drawpivot(coord_t, coord_t);
void drawbox(coord_t, coord_t, coord_t, coord_t, int);
void erasebox(coord_t, coord_t, coord_t, coord_t);
void showcoord(const bool);
void showrules(void);
void benchmark(void);
void get_rnd_density();
void randomize(void);
void settimeout(unsigned long);
void color_button(int, int, int);
void setcolor(int, unsigned long, unsigned long);
void wait_activity(void);

/* UNIX interface */
#define SYSERR strerror(errno)

/* X I/O state information */
extern Display *disp;
extern Window rootw, mainw, lifew, helpw, inputw, coordw, rulew;
extern int screen;
extern unsigned long fcolor, bcolor;
extern XEvent event;
extern XFontStruct *nfont, *bfont;
extern GC ntextgc, btextgc, inputgc, cellgc[], xorgc;
extern KeySym ks;

#define INPBUFLEN 255
extern char inpbuf[INPBUFLEN];
extern int minbuflen;
#define DIRBUFLEN 100
extern int numcomments;
#define MAXCOMMENTS 256
extern char comments[MAXCOMMENTS][256];
extern char loadirbuf[DIRBUFLEN];
extern char keybuf[16];
extern char lookup[0xfffff];
extern char fname[256];

#if STATEBITS > 1
extern unsigned int maxstates;
#endif /* STATEBITS > 1 */

extern coord_t xpos, ypos;       /* coords of upper left corner of viewport */
extern coord_t xorigin, yorigin; /* virtual origin */
extern coord_t lastx, lasty;     /* last coord pair displayed on status line */
extern coord_t loadx, loady;     /* location to load new points */
extern coord_t iloadx, iloady;   /* initial location of load points */
extern int txx, txy, tyx, tyy;   /* transform for tentative points */

extern bool dispboxes;
extern long scale;
extern unsigned long born;
extern unsigned long live;

extern int width;
extern int height;
extern int inputlength;
extern int state;
extern bool dispcoord;
extern struct timeval timeout;

#if STATEBITS == 1
#define paintcolor 1
#else
extern int paintcolor;
extern char active_rules[100];
#endif

extern int inputwwidth;
#ifdef PROF
extern int link_called;
extern int link_search;
extern int create_called;
extern int create_null;
#endif /* PROF */

extern char saveformat;
extern pattern active, tentative;       /* the two pattern layers */
extern tile *freep;                     /* free-tile list */

#define MAXCHANGE       8192
extern cellcount_t chgpoints[MAXSTATES], chgrects[MAXSTATES];
extern XPoint points[MAXSTATES][MAXCHANGE];
extern XRectangle rects[MAXSTATES][MAXCHANGE];

/* cell.c */
int getcell(cellbox*, const int, const int);
void setcell(cellbox*, const int, const int, const int);
void forget(cellbox*);
void displaybox(u32bits, u32bits, cellbox*);
void trdisplaybox(u32bits, u32bits, cellbox*);
void trdrawbox(coord_t, coord_t, coord_t, coord_t, int);

/* generate.c */
void generate(pattern *);
void newrules(void);
char *readrules(const char *);
char *parse_rule(char*);

/* tile.c */
void setscale(const int);
void changesize(const int);
void initcells(pattern*);
void chgcell(pattern*, coord_t, coord_t, cell_t);
void killtile(pattern*, tile*);
int lookcell(pattern*, coord_t, coord_t);
tile *fetchtile(pattern*, coord_t, coord_t);
tile *maketile(pattern*, coord_t, coord_t);
void center(void);
void displaystats(void);
void clear_pattern(pattern *);
void clear(void);
void redisplay(void);
void redrawscreen(void);
void stamp(char*, FILE*);
cellcount_t bounding_box(pattern*);
void saveall(FILE*, char);
pattern *analyze(pattern*);
void savesimple(pattern*, int, int, FILE*);
cellcount_t cellcnt(pattern*);

extern tile *curtile;
extern char matchlibfile[];

#define FOR_CELLS(pp, pt, dx, dy)       for (pt=(pp)->tiles;pt;pt=pt->next) \
                                            if (!pt->dead) \
                                                for (dx=0;dx<BOXSIZE;dx++) \
                                                    for (dy=0;dy<BOXSIZE;dy++)
extern int oldp, qdirs;

#define MAXDIRS         64
extern char *dirs[];

char *checktilda(char*);
char *addlifeext(char*);
char *seppatdir(char*, char*);
void parse_patname(char*, char*);

extern unsigned delay;
extern char stashed[];
extern char outcome[];
extern int convmode, wireframe, dispmesh, showgrid;

#define MAXCOLORS (MAXSTATES+3)
#define LOAD_BOX MAXSTATES
#define GRID_COLOR (MAXSTATES+1)
#define GRID_ECOLOR (MAXSTATES+2)
